#include <stdio.h>
#include <stdlib.h>
#include <string.h>
//#include <unistd.h>
#include <assert.h>


struct node
    {
    struct node *next;
    char *name;
    int length;
    };

int getChrLength(struct node *sl, char *what)
{
for(; sl!=NULL; sl=sl->next)
    {
    if(strcmp(sl->name, what) == 0)
        return sl->length;
    }
return 0;
}



int main(int argc, char *argv[])
{

if(argc != 5)
    {
    printf("Usage: %s\n"
    	"\t1. <ucsc gene data (refGene.txt, etc)>\n"
    	"\t2. <[geneType], \"refGene\", ...>\n"
	"\t3. <chr size file (be complete with contigs)>\n"
	"\t4. <0/1, whether first field is bin>\n"
        "output files (written to current directory)\n"
        "\tpromoter.bigBed, utr5.bigBed, utr3.bigBed, exons.bigBed, introns.bigBed, [geneType].bigBed, [geneType].struct.txt\n"
	"Note:\n"
	"\t[geneType] could be knownGene, refGene, ... input file format\n"
	"\tin this file, following fields must be next to each other: name, chrom, strand, txStart, txEnd, cdsStart, cdsEnd, exonCount, exonStarts, exonEnds\n"
	"\t[geneType].bigBed contains gene body position\n"
	"\t[geneType].bigBed shares id with [geneType].struct.txt file\n", argv[0]);
    exit(0);
    }

char *chrSizeFile = argv[3];

// load chr size file
struct node *chrSl = NULL;
FILE *fin = fopen(chrSizeFile, "r");
if(fin == NULL)
    {
    printf("%m: %s\n", chrSizeFile);
    exit(0);
    }
size_t s = 1;
char *line = malloc(1), *tok;
char delim[] = "\t\n";
while(getline(&line, &s, fin) != -1)
    {
    tok = strtok(line, delim);
    struct node *this = malloc(sizeof(struct node));
    asprintf(&this->name, "%s", tok);
    this->length = strtol(strtok(NULL, delim), NULL, 0);
    this->next = chrSl;
    chrSl = this;
    }
fclose(fin);


int hasBin = strtol(argv[4], NULL, 0);
char *geneType = argv[2];

char *inFile = argv[1];
fin = fopen(inFile, "r");
if(fin == NULL)
    {
    printf("%m: %s\n", inFile);
    exit(0);
    }

FILE *promoterF = fopen("promoter", "w");
FILE *utr5F = fopen("utr5", "w");
FILE *utr3F = fopen("utr3", "w");
FILE *exonF = fopen("exons", "w");
FILE *intronF = fopen("introns", "w");


char *genebodyBedFile;
asprintf(&genebodyBedFile, "%s", geneType);
FILE *gbF = fopen(genebodyBedFile, "w");
char *geneStructFile;
asprintf(&geneStructFile, "%s.struct.txt", geneType);
FILE *gbstructF = fopen(geneStructFile, "w");
int pI=0, // promoter id
    u5I=0, // 5' utr id
    u3I=0, // 3' utr id
    eI=0, // exon id
    iI=0, // intron id
    gI=0; // gene id, for both gbF and gbstructF


/*
  `bin` smallint(5) unsigned NOT NULL default '0',
  `name` varchar(255) NOT NULL default '',
  `chrom` varchar(255) NOT NULL default '',
  `strand` char(1) NOT NULL default '',
  `txStart` int(10) unsigned NOT NULL default '0',
  `txEnd` int(10) unsigned NOT NULL default '0',
  `cdsStart` int(10) unsigned NOT NULL default '0',
  `cdsEnd` int(10) unsigned NOT NULL default '0',
  `exonCount` int(10) unsigned NOT NULL default '0',
  `exonStarts` longblob NOT NULL,
  `exonEnds` longblob NOT NULL,

  following fields are not used:

  `id` int(10) unsigned NOT NULL default '0',
  `name2` varchar(255) NOT NULL default '',
  `cdsStartStat` enum('none','unk','incmpl','cmpl') NOT NULL default 'none',
  `cdsEndStat` enum('none','unk','incmpl','cmpl') NOT NULL default 'none',
  `exonFrames` longblob NOT NULL,
 */

char delim2[] = ",";
while(getline(&line, &s, fin) != -1)
    {
    // name
    char *name = strtok(line, delim);
    if(hasBin)
        name = strtok(NULL, delim);
    assert(name!=NULL);
    // chromosome
    char *chr = strtok(NULL, delim); assert(chr!=NULL);
    int chrLen = getChrLength(chrSl, chr);
    assert(chrLen != 0);
    // strand
    tok = strtok(NULL, delim); assert(tok!=NULL);
    char strand = tok[0];
    // tx start
    tok = strtok(NULL, delim); assert(tok!=NULL);
    int txStart = strtol(tok, NULL, 0);
    // tx stop
    tok = strtok(NULL, delim); assert(tok!=NULL);
    int txStop = strtol(tok, NULL, 0);
    // cds start
    tok = strtok(NULL, delim); assert(tok!=NULL);
    int cdsStart = strtol(tok, NULL, 0);
    // cds stop
    tok = strtok(NULL, delim); assert(tok!=NULL);
    int cdsStop = strtol(tok, NULL, 0);
    // exon count
    tok = strtok(NULL, delim); assert(tok!=NULL);
    int exonCount = strtol(tok, NULL, 0);
    // exon starts
    char *exonstarts = strtok(NULL, delim); assert(exonstarts!=NULL);
    // exon stops
    char *exonstops = strtok(NULL, delim); assert(exonstops!=NULL);


    /**** OUTPUT *****/
    // gene body
    fprintf(gbF, "%s\t%d\t%d\t%s\t%d\t%c\n", chr, txStart, txStop, name, gI,strand);
    fprintf(gbstructF, "%d\t%s\t%c\t%d\t%d\t%d\t%d\t%d\t%s\t%s\t%s\n", gI++, chr, strand, txStart, txStop, cdsStart, cdsStop, exonCount, exonstarts, exonstops, name);

    // promoter
    if(strand == '+')
        {
	if(txStart > 0)
            fprintf(promoterF, "%s\t%d\t%d\t%s\t%d\t+\n", chr, (txStart > 3000) ? txStart-3000 : 0, txStart, name, pI++);
	}
    else
        {
	if(txStop < chrLen)
            fprintf(promoterF, "%s\t%d\t%d\t%s\t%d\t-\n", chr, txStop, (txStop+3000>chrLen)? chrLen-1 : txStop+3000, name, pI++);
	}

    // don't process non-coding genes for exons/introns/UTRs
    if(cdsStart == cdsStop)
        {
	continue;
	}

    // exons, introns
    int exonStartCoord[exonCount], exonStopCoord[exonCount];
    int i;
    exonStartCoord[0] = strtol(strtok(exonstarts, delim2), NULL, 0);
    for(i=1; i<exonCount; i++)
	exonStartCoord[i] = strtol(strtok(NULL, delim2), NULL, 0);
    exonStopCoord[0] = strtol(strtok(exonstops, delim2), NULL, 0);
    for(i=1; i<exonCount; i++)
	exonStopCoord[i] = strtol(strtok(NULL, delim2), NULL, 0);
    if(exonStartCoord[0] < exonStopCoord[0])
        fprintf(exonF, "%s\t%d\t%d\t%s\t%d\t%c\n", chr, exonStartCoord[0], exonStopCoord[0], name, eI++, strand);
    int intronStart = exonStopCoord[0];
    for(i=1; i<exonCount; i++)
        {
	if(exonStartCoord[i] < exonStopCoord[i])
            fprintf(exonF, "%s\t%d\t%d\t%s\t%d\t%c\n", chr, exonStartCoord[i], exonStopCoord[i], name, eI++, strand);
	if(intronStart < exonStartCoord[i])
	    fprintf(intronF, "%s\t%d\t%d\t%s\t%d\t%c\n", chr, intronStart, exonStartCoord[i], name, iI++, strand);
	intronStart = exonStopCoord[i];
	}

    // 5' 3' utr
    if(strand == '+')
	{
	int thisnum = (cdsStart > exonStopCoord[0]) ? exonStopCoord[0] : cdsStart;
	if(thisnum > txStart)
	    fprintf(utr5F, "%s\t%d\t%d\t%s\t%d\t+\n", chr, txStart, thisnum, name, u5I++);
	thisnum = (exonStartCoord[exonCount-1] > cdsStop) ? exonStartCoord[exonCount-1] : cdsStop;
	if(thisnum < txStop)
	    fprintf(utr3F, "%s\t%d\t%d\t%s\t%d\t+\n", chr, thisnum, txStop, name, u3I++);
	}
    else
	{
	int thisnum = (exonStartCoord[exonCount-1] > cdsStop) ? exonStartCoord[exonCount-1] : cdsStop;
	if(thisnum < txStop)
	    fprintf(utr5F, "%s\t%d\t%d\t%s\t%d\t-\n", chr, thisnum, txStop, name, u5I++);
	thisnum = (cdsStart > exonStopCoord[0]) ? exonStopCoord[0] : cdsStart;
	if(thisnum > txStart)
	    fprintf(utr3F, "%s\t%d\t%d\t%s\t%d\t-\n", chr, txStart, thisnum, name, u3I++);
	}
    }
fclose(promoterF);
fclose(utr5F);
fclose(utr3F);
fclose(exonF);
fclose(intronF);
fclose(gbF);
fclose(gbstructF);


char *com;
system("bedSort promoter promoter");
//asprintf(&com, "bedToBigBed promoter.bed %s %spromoter.bigBed", chrSizeFile, geneType);
asprintf(&com, "bgzip promoter -c > %spromoter.gz", geneType);
system(com);
asprintf(&com, "tabix -p bed %spromoter.gz", geneType);
system(com);
unlink("promoter");

system("bedSort utr5 utr5");
//asprintf(&com, "bedToBigBed utr5.bed %s %sutr5.bigBed", chrSizeFile, geneType);
asprintf(&com, "bgzip utr5 -c > %sutr5.gz", geneType);
system(com);
asprintf(&com, "tabix -p bed %sutr5.gz", geneType);
system(com);
unlink("utr5");

system("bedSort utr3 utr3");
asprintf(&com, "bgzip utr3 -c > %sutr3.gz", geneType);
system(com);
asprintf(&com, "tabix -p bed %sutr3.gz", geneType);
system(com);
unlink("utr3");

system("bedSort exons exons");
//asprintf(&com, "bedToBigBed exons.bed %s %sexons.bigBed", chrSizeFile, geneType);
asprintf(&com, "bgzip exons -c > %sexons.gz", geneType);
system(com);
asprintf(&com, "tabix -p bed %sexons.gz", geneType);
system(com);
unlink("exons");

system("bedSort introns introns");
//asprintf(&com, "bedToBigBed introns.bed %s %sintrons.bigBed", chrSizeFile, geneType);
asprintf(&com, "bgzip introns -c > %sintrons.gz", geneType);
system(com);
asprintf(&com, "tabix -p bed %sintrons.gz", geneType);
system(com);
unlink("introns");

asprintf(&com, "bedSort %s %s", genebodyBedFile, genebodyBedFile);
system(com);
//asprintf(&com, "bedToBigBed %s %s %s.bigBed", genebodyBedFile, chrSizeFile, geneType);
asprintf(&com, "bgzip %s -c > %s.gz", genebodyBedFile, genebodyBedFile);
system(com);
asprintf(&com, "tabix -p bed %s.gz", genebodyBedFile);
system(com);
unlink(genebodyBedFile);

printf("drop table if exists %sstruct;\n"
"create table %sstruct (\n"
  "id int unsigned not null primary key,\n"
  "chrom varchar(255) not null,\n"
  "strand char(1) not null,\n"
  "txStart int unsigned not null,\n"
  "txEnd int unsigned not null,\n"
  "cdsStart int unsigned not null,\n"
  "cdsEnd int unsigned not null,\n"
  "exonCount int unsigned not null,\n"
  "exonStarts text not null,\n"
  "exonEnds text not null,\n"
  "name varchar(255) not null\n"
");\n"
"load data local infile '%s' into table %sstruct;\n", geneType, geneType, geneStructFile, geneType);

}
